@@ -1233,8 +1233,30 @@ out:
     return ret;
 }
 
+/**
+ * gnome_rr_screen_clear_dpms_timeouts:
+ **/
+static gboolean
+gnome_rr_screen_clear_dpms_timeouts (GnomeRRScreen *screen,
+                                     GError **error)
+{
+    gdk_error_trap_push ();
+    /* DPMSSetTimeouts() return value is often a lie, so ignore it */
+    DPMSSetTimeouts (screen->priv->xdisplay, 0, 0, 0);
+    if (gdk_error_trap_pop ()) {
+        g_set_error_literal (error,
+                             GNOME_RR_ERROR,
+                             GNOME_RR_ERROR_UNKNOWN,
+                             "Could not set DPMS timeouts");
+        return FALSE;
+    }
+    return TRUE;
+}
+
 /**
  * gnome_rr_screen_set_dpms_mode:
+ *
+ * This method also disables the DPMS timeouts.
  **/
 gboolean
 gnome_rr_screen_set_dpms_mode (GnomeRRScreen *screen,
@@ -1251,8 +1273,10 @@ gnome_rr_screen_set_dpms_mode (GnomeRRScreen *screen,
     ret = gnome_rr_screen_get_dpms_mode (screen, &current_mode, error);
     if (!ret)
         goto out;
-    if (current_mode == mode)
+    if (current_mode == mode) {
+        ret = gnome_rr_screen_clear_dpms_timeouts (screen, error);
         goto out;
+    }
 
     switch (mode) {
     case GNOME_RR_DPMS_ON:
@@ -1273,12 +1297,9 @@ gnome_rr_screen_set_dpms_mode (GnomeRRScreen *screen,
     }
 
     gdk_error_trap_push ();
-    ret = DPMSForceLevel (screen->priv->xdisplay, state);
-    gdk_flush ();
-    if (gdk_error_trap_pop ())
-        ret = FALSE;
-
-    if (!ret) {
+    /* DPMSForceLevel() return value is often a lie, so ignore it */
+    DPMSForceLevel (screen->priv->xdisplay, state);
+    if (gdk_error_trap_pop ()) {
         ret = FALSE;
         g_set_error_literal (error,
                              GNOME_RR_ERROR,
@@ -1286,6 +1307,10 @@ gnome_rr_screen_set_dpms_mode (GnomeRRScreen *screen,
                              "Could not change DPMS mode");
         goto out;
     }
+
+    ret = gnome_rr_screen_clear_dpms_timeouts (screen, error);
+    if (!ret)
+        goto out;
 out:
     return ret;
 }
@@ -1538,10 +1563,11 @@ update_brightness_limits (GnomeRROutput *output)
     atom = XInternAtom (DISPLAY (output), "BACKLIGHT", FALSE);
     info = XRRQueryOutputProperty (DISPLAY (output), output->id, atom);
     rc = gdk_error_trap_pop ();
-    if (rc != 0)
+    if (rc != Success)
     {
-        g_warning ("could not get output property for %s, rc: %i",
-		   output->name, rc);
+        if (rc != BadName)
+          g_warning ("could not get output property for %s, rc: %i",
+		     output->name, rc);
         goto out;
     }
     if (info == NULL)
@@ -1558,7 +1584,10 @@ update_brightness_limits (GnomeRROutput *output)
     output->backlight_min = info->values[0];
     output->backlight_max = info->values[1];
 out:
-    XFree (info);
+    if (info != NULL)
+    {
+        XFree (info);
+    }
 #endif
 }
 
